/* Device Info
 * This source is used to parse a PCI device database in order to find:
 * - Vendor Name
 * - Device Name
 * - Kernel Module
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <badpenguin-fs.h>
#include "device-manager.h"

int find_char(char *s, char c) {
  int p=0;
  while (s[p]!=c) p++;
  if (s[p]==c) {
    return p; 
  } else {
    return -1;
  }
}

int parse_info_line(TPciDevice *infodevice, char *line) {
  char *templine = line;
  char *destline = NULL;
  int l=0;

  // skip vendor.device ID
  templine= templine + 10;
  // find module
  l = find_char(templine,'|');
  if (l>=0) {
    destline = (char*) calloc(l+1, sizeof(char));
    strncpy(destline, templine, l);
    infodevice->kernelModule = strdup(destline);
  } else
    return -1;
  // find vendor
  templine= templine + l + 1;
  l = find_char(templine,'|');
  if (l>=0) {
    destline = (char*) calloc(l+1, sizeof(char));
    strncpy(destline, templine, l);
    infodevice->vendorName = strdup(destline);
  } else
    return -1;
  // find device description
  templine= templine + l + 1;
  infodevice->deviceName = strdup(templine);
  return 0;
}

unsigned long char2hex(char c) {
  if (c=='0') return 0;
  if ((c>='1')&&(c<='9')) return (c-'1')+1;
  if ((c>='a')&&(c<='f')) return (c-'a')+10;
  if ((c>='A')&&(c<='F')) return (c-'A')+10;
  assert(0!=1);
  return 0;
}

unsigned long getdword2(unsigned char *base) {
  unsigned long count;
  count = char2hex(base[3]);
  count += ( char2hex(base[2]) * 16 );
  count += ( char2hex(base[1]) * 16 * 16 );
  count += ( char2hex(base[0]) * 16 * 16 * 16 );
  return count;
}

int check_device_info(TPciDevice *devicepci, char *detectfile) {
  FILE	*fd = NULL;
  char  line[256];
  int   infofound= 0;
  int   retcode = 0;
  
  fd = fopen(detectfile,"r");
  if (fd==NULL) {
    printf("! Unable to open detect list %s \n", detectfile);
    return -1;
  }
  while ( ! feof(fd) ) {
    read_line(fd, line);
    if ( ( getdword2(line) == devicepci->vendor ) && ( getdword2(line+5) == devicepci->device ) ) {
      infofound++;
      devicepci->genericModule = 2;
      if (giVerbose>2) printf(">%s\n", line);
      break;
    }
    
  }
  if (infofound==0) {
    rewind(fd);
    while ( ! feof(fd) ) {
      read_line(fd, line);
      if ( ( getdword2(line) == devicepci->vendor ) && (0 == strncasecmp(line+5,"XXXX",4)) ) {
        infofound++;
	devicepci->genericModule = 1;
        retcode = 1;
        if (giVerbose>2) printf(">%s\n", line);
        break;
      }    
    }    
  }
  fclose(fd);
  if (infofound==0) {
    devicepci->genericModule = 3;
    printf("! No info found about the device %4x.%4x\n",devicepci->vendor,devicepci->device);
    printf("  Please send to agx@linux.it the information\n");
    printf("  about the /proc/pci file. Thank you, AGX. \n");
    sleep(5);
    return -1;
  }
  // Parse Detect Line
  parse_info_line(devicepci,line);
  // Show Info
  if (giVerbose>0) {  
    printf("scanpci: %4x.%4x %s \n", devicepci->vendor, devicepci->device, devicepci->kernelModule );
    printf("       : %s %s", devicepci->vendorName, devicepci->deviceName );
    if (devicepci->genericModule==1) printf(" (Generic Module)");
    printf("\n");
  }    
  return retcode;
}
